"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DIProvider = void 0;
const jsx_runtime_1 = require("react/jsx-runtime");
const cheap_di_1 = require("cheap-di");
const react_1 = require("react");
const DiContext_js_1 = require("../DiContext.js");
const index_js_1 = require("../hooks/index.js");
const InternalLogger_js_1 = require("../InternalLogger.js");
const DIProvider = (props) => {
    const { dependencies, self, selfSingletons, parentContainer, debugName, children } = props;
    const [logger] = (0, react_1.useState)(() => new InternalLogger_js_1.InternalLogger(debugName));
    const [initialized, setInitialized] = (0, react_1.useState)(false);
    const timerRef = (0, react_1.useRef)(null);
    const diContext = (0, index_js_1.useDiContext)({ logger, parentContainer });
    const { container } = diContext;
    (0, react_1.useEffect)(() => {
        const isAnyConfigurationPassed = dependencies || self || selfSingletons || parentContainer;
        if (!container || !isAnyConfigurationPassed) {
            return;
        }
        container.clear();
        logger.log('dependency registrations');
        const singletonsSizeBeforeDependenciesUpdate = container?.getSingletons().size ?? 0;
        dependencies?.forEach((dependency) => dependency(container));
        self?.forEach((selfDependency) => container.registerImplementation(selfDependency));
        selfSingletons?.forEach((selfDependency) => container.registerImplementation(selfDependency).asSingleton());
        logger.log('singleton and stateful configurations');
        for (const [type] of container.getDependencies()) {
            const constructor = container.localScope((_container) => _container.getImplementation(type));
            if (!constructor) {
                return;
            }
            if ((0, cheap_di_1.isSingleton)(constructor)) {
                logger.log('singleton', constructor, 'founded');
                // resolve type to get and register its instance
                container.resolve(type);
            }
        }
        if (container.parentContainer && singletonsSizeBeforeDependenciesUpdate !== container.getSingletons().size) {
            logger.log('singletons size changed, trigger root rerender');
            timerRef.current = setTimeout(() => {
                container.rootContainer.rerender();
            });
        }
        setInitialized(true);
    }, [container, dependencies, self]);
    (0, react_1.useEffect)(() => {
        return () => {
            if (timerRef.current) {
                clearTimeout(timerRef.current);
            }
            container?.clear();
        };
    }, []);
    if (!initialized) {
        return null;
    }
    return ((0, jsx_runtime_1.jsx)(DiContext_js_1.DiContext.Provider, { value: diContext, children: (0, jsx_runtime_1.jsx)(MemoizedChildren, { children: children }) }));
};
const MemoizedChildren = (0, react_1.memo)(({ children }) => (0, jsx_runtime_1.jsx)(react_1.Fragment, { children: children }));
const memoizedComponent = (0, react_1.memo)(DIProvider);
exports.DIProvider = memoizedComponent;
//# sourceMappingURL=DIProvider.js.map